from google import genai
from google.genai import types
import random
import os
import json
import time
import tqdm

client = genai.Client(api_key=os.environ["GEMINI_API_KEY"])

prompt_dir = 'prompts\\cot_analysis'
output_dir = 'results\\gemini'
summeval_dir = 'data\\summeval.json'
model = 'gemini-2.0-flash'

os.makedirs(output_dir, exist_ok=True)
summeval = json.load(open(summeval_dir))

random.seed(42)
sample_indices = sorted(random.sample(range(len(summeval)), 100))
sample_summeval = [summeval[i] for i in sample_indices]

for file in os.listdir(prompt_dir):
    if file.endswith(".txt"):
        prompt_fp = os.path.join(prompt_dir, file)
        output_fp = os.path.join(output_dir, file.replace(".txt", ".json"))
        prompt = open(prompt_fp, "r").read()
        
        ct, ignore = 0, 0
        
        new_json = []
        for instance in tqdm.tqdm(sample_summeval):
            source = instance['source']
            system_output = instance['system_output']
            cur_prompt = prompt.replace('{{Document}}', source).replace('{{Summary}}', system_output)
            instance['prompt'] = cur_prompt
            while True:
                try:
                    _response = client.models.generate_content(
                        model=model,
                        contents=cur_prompt,
                        config=types.GenerateContentConfig(
                        temperature=2,
                        max_output_tokens=5,    #Should have been slightly higher to accommodate for invalid model responses
                        top_p=1,
                        frequency_penalty=0,
                        presence_penalty=0,
                        stop_sequences=None,
                        #Gemini does not support the 'n' and logprobs parameter, therefore it's inefficient to use higher temperatures for log probability calculations
                        )
                    )
                    time.sleep(0.5)

                    instance['model_response'] = _response.text
                    new_json.append(instance)
                    ct += 1

                    if ct >= 15:           #Circumvent rate error due to limitation of 15 requests per minute
                        time.sleep(60)
                        ct=0
                        
                    break
                except Exception as e:
                    print(e)
                    if ("limit" in str(e)):
                        time.sleep(2)
                    else:
                        ignore += 1
                        print('ignored', ignore)
                        break

        print('ignored total', ignore)
        with open(output_fp, 'w') as f:
            json.dump(new_json, f, indent=4)