from openai import OpenAI
import random
import os
import json
import tqdm
import time

client = OpenAI(api_key=os.environ["HYPERBOLIC_API_KEY"], base_url="https://api.hyperbolic.xyz/v1",)

prompt_dir = 'prompts\\cot_analysis'
output_dir = 'results\\llama'
summeval_dir = 'data\\summeval.json'
model = 'meta-llama/Meta-Llama-3-70B-Instruct'

os.makedirs(output_dir, exist_ok=True)
summeval = json.load(open(summeval_dir))

random.seed(42)
sample_indices = sorted(random.sample(range(len(summeval)), 100))
sample_summeval = [summeval[i] for i in sample_indices]

for file in os.listdir(prompt_dir):
    if file.endswith(".txt"):
        prompt_fp = os.path.join(prompt_dir, file)
        output_fp = os.path.join(output_dir, file.replace(".txt", ".json"))
        prompt = open(prompt_fp, "r").read()
        
        ct, ignore = 0, 0
        
        new_json = []
        for instance in tqdm.tqdm(sample_summeval):
            source = instance['source']
            system_output = instance['system_output']
            cur_prompt = prompt.replace('{{Document}}', source).replace('{{Summary}}', system_output)
            instance['prompt'] = cur_prompt
            while True:
                try:
                    _response = client.chat.completions.create(
                        model=model, 
                        messages=[
                            {"role": "system", "content": "You are a helpful assistant that compares the Source Text and Summary based on the prompt given. "
                            "You only give numerical values as response, based on the Evalation Steps given, and do not respond with text."},
                            {"role": "user", "content": cur_prompt}
                            ], 
                        temperature=0,
                        max_completion_tokens=10,
                        top_p=1,
                        frequency_penalty=0,
                        presence_penalty=0,
                        stop=None,
                        #logprobs=True,     Not yet supported
                        #n=20               Possible to estimate logprobs via high temperature and n
                    )
                    time.sleep(0.5)

                    instance['model_response'] = _response.choices[0].message.content
                    new_json.append(instance)
                    ct += 1
                    break
                except Exception as e:
                    print(e)
                    if ("limit" in str(e)):
                        time.sleep(2)
                    else:
                        ignore += 1
                        print('ignored', ignore)

                        break

        print('ignored total', ignore)
        with open(output_fp, 'w') as f:
            json.dump(new_json, f, indent=4)