from prettytable import PrettyTable
from scipy.stats import spearmanr, pearsonr, kendalltau
import json
import re
import os
import argparse
from glob import glob

def calculate_correlation(pred_score, human_score, result):
    assert len(pred_score) == len(human_score)

    #Throws error: Part of original code, not sure how to fix without breaking it
    if (len(result) == 0) or result is None:
        result = {'pearson': 0, 'spearman': 0, 'kendalltau': 0}
    result['pearson'] += pearsonr(pred_score, human_score)[0]
    result['spearman'] += spearmanr(pred_score, human_score)[0]
    result['kendalltau'] += kendalltau(pred_score, human_score)[0]

    return result

#Problematic approach, but wasn't sure how else to do it (alternative is changing the filenames to name the dimensions directly)
def dimension_from_filename(filename):

    #Asked ChatGPT for this Regex
    matched = re.search(r'_(\w+?)(?=_|\b)', filename)
    if matched:
        abbreviation = matched.group(1).lower()
        
        dimension_mapping = {
        'coh': 'coherence',
        'con': 'consistency',
        'rel': 'relevance',
        'flu': 'fluency'
        }

        return dimension_mapping.get(abbreviation, abbreviation)
    else:
        return None

def print_correlations(result, n, input_file, dimension, avg_pred_score, skipped_scores, output_file):
    table = PrettyTable(['Pearson', 'Spearman', 'Kendall'])
    if (n == 0):
        n = 1
    table.add_row(
        [round(result['pearson'] / n, 4), round(result['spearman'] / n, 4), round(result['kendalltau'] / n, 4)])
    
    output_file.write(f"\nFilename: {input_file}, Dimension: {dimension}, Average Prediction Score: {avg_pred_score}, Skipped Scores: {skipped_scores}\n")
    output_file.write(str(table))

def parse_output(output):
    #Asked ChatGPT for this Regex (changed it from original code to capture model responses that do not start with a numerical value)
    matched = re.search(r"([1-5]+(?:\.[1-5]*)?)", output)
    if matched:
        score = float(matched.group(1))
    else:
        score = 0
    return score

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--input_folder', type=str, required=True)
    parser.add_argument('--output_filename', type=str, required=True) 
    args = parser.parse_args()

    files = glob(os.path.join(args.input_folder, "*.json"))
    
    output_dir = os.path.join(os.getcwd(), "correlation_tables")
    if not os.path.exists(output_dir):
        os.makedirs(output_dir)

    output_filepath = os.path.join(output_dir, args.output_filename)

    with open(output_filepath, 'w') as output_file:
        for input_file in files:
            dimension = dimension_from_filename(os.path.basename(input_file))
            if dimension:
                with open(input_file, 'r') as f:
                    file = json.load(f)
                    
                pred_scores, human_scores, skipped_scores = {}, {}, 0

                for item in file:
                    doc_id = item["doc_id"]
                    if (doc_id not in pred_scores):
                        pred_scores[doc_id] = []
                        human_scores[doc_id] = []
                    
                    model_response = item["model_response"]
                    score = parse_output(model_response)

                    #Filtering of invalid model responses (sometimes the maximum token value restricts a numerical value from being in the response)
                    if score == 0:
                        skipped_scores += 1
                        continue

                    pred_scores[doc_id].append(score)
                    human_scores[doc_id].append(item['scores'][dimension])

                print(f'Skipped scores due to invalid model response: {skipped_scores}')
                results = {'pearson': 0, 'spearman': 0, 'kendalltau': 0}
                total_pred_score, scores_count, d_ctr = 0, 0, 0
                
                for doc_id in pred_scores:
                    pred_scores_doc = pred_scores[doc_id]
                    human_scores_doc = human_scores[doc_id]
                    if (len(set(human_scores_doc)) <= 1) or (len(set(pred_scores_doc)) <= 1):
                        continue

                    results = calculate_correlation(pred_scores_doc, human_scores_doc, results)
                    total_pred_score += sum(pred_scores_doc)
                    scores_count += len(pred_scores_doc)
                    d_ctr += 1

                avg_pred_score = total_pred_score / scores_count
                print_correlations(results, n=d_ctr, input_file = input_file, dimension = dimension, avg_pred_score = avg_pred_score, skipped_scores = skipped_scores, output_file = output_file)
            else:
                print(f"Skipping file: {input_file}")